package pathfinder

import (
	"fmt"
	"reflect"
	"strings"
)

func FindPaths(v reflect.Value, targetType reflect.Type, path []string, paths *[]string) {

	if v.Kind() == reflect.Invalid {
		return
	}

	vType := v.Type()

	switch v.Kind() {
	case reflect.Ptr:
		FindPaths(v.Elem(), targetType, path, paths)
	case reflect.Struct:
		for i := 0; i < v.NumField(); i++ {
			newPath := append(path, vType.Field(i).Name)
			FindPaths(v.Field(i), targetType, newPath, paths)
		}
	case reflect.Map:
		for _, key := range v.MapKeys() {
			newPath := append(path, fmt.Sprint(key))
			FindPaths(v.MapIndex(key), targetType, newPath, paths)
			if v.MapIndex(key).Type() == targetType {
				*paths = append(*paths, strings.Join(newPath, "."))
			}
		}
	case reflect.Slice, reflect.Array:
		for i := 0; i < v.Len(); i++ {
			newPath := append(path, fmt.Sprint(i))
			FindPaths(v.Index(i), targetType, newPath, paths)
			if v.Index(i).Type() == targetType {
				*paths = append(*paths, strings.Join(newPath, "."))
			}
		}
	case reflect.String:
		if vType != targetType {
			return
		}

	default:
		return
	}

	if vType == targetType {
		*paths = append(*paths, strings.Join(path, "."))
	}
}
